#ifndef DECOMPOSINGDELEGATEADAPTER_H
#define DECOMPOSINGDELEGATEADAPTER_H

#include "Creator.h"
#include "Dispatcher.h"
#include "Format.h"
#include "FormatDecomposer.h"
#include "FormatVisitor.h"

class DecomposingDelegateAdapter : public Delegate, public Creator
{
protected:
  class DelegationVisitor : public ConstFormatVisitor
  {
  protected:
    DecomposingDelegateAdapter* adapter_;

  public:
    DelegationVisitor(DecomposingDelegateAdapter* adapter) : adapter_(adapter) { }

    virtual ~DelegationVisitor() { }

    virtual void visit(const Format* fmt) { adapter_->delegate_->dispatch(fmt->clone(adapter_)); }
  };

protected:
  Delegate* delegate_;

  FormatDecomposer* decomposer_;

  DelegationVisitor visitor_;

public:
  DecomposingDelegateAdapter(Delegate* delegate, FormatDecomposer* decomposer) : delegate_(delegate), decomposer_(decomposer), visitor_(this) { }

  virtual ~DecomposingDelegateAdapter() { }

  virtual void dispatch(Format* fmt)
  {
    decomposer_->decompose(fmt, visitor_);
    fmt->getCreator()->deallocate(fmt);
  }

  virtual void run(size_t limit, size_t max_num_live_pipelines)
  {
    delegate_->run(limit, max_num_live_pipelines);
  }
};

#endif // DECOMPOSINGDELEGATEADAPTER_H
