#ifndef EXCEPTIONS_H
#define EXCEPTIONS_H

#include <string>

class Exception
{
protected:
  std::string msg_;
public:
  Exception() { }
  Exception(const std::string& msg) : msg_(msg) { }
  std::string getMessage() const { return msg_; }
};


// General exceptions

class NotImplementedException : public Exception
{
public:
  NotImplementedException() { }
  NotImplementedException(const std::string& msg) : Exception(msg) { }
};

class OutOfBoundsException : public Exception
{
public:
  OutOfBoundsException() { }
  OutOfBoundsException(const std::string& msg) : Exception(msg) { }
};

class EntryExistsException : public Exception
{
public:
  EntryExistsException() { }
  EntryExistsException(const std::string& msg) : Exception(msg) { }
};

class NoEntryExistsException : public Exception
{
public:
  NoEntryExistsException() { }
  NoEntryExistsException(const std::string& msg) : Exception(msg) { }
};

// Format exceptions

class InvalidTypeException : public Exception
{
public:
  InvalidTypeException() { }
  InvalidTypeException(const std::string& msg) : Exception(msg) { }
};

class UndefinedFieldException : public Exception
{
public:
  UndefinedFieldException() { }
  UndefinedFieldException(const std::string& msg) : Exception(msg) { }
};

class UnterminatedFieldException : public Exception
{
public:
  UnterminatedFieldException() { }
  UnterminatedFieldException(const std::string& msg) : Exception(msg) { }
};

class SizeException : public Exception
{
public:
  SizeException() { }
  SizeException(const std::string& msg) : Exception(msg) { }
};


// Source / Classifier Exceptions

class UnsupportedModeException : public Exception
{
public:
  UnsupportedModeException() { }
  UnsupportedModeException(const std::string& msg) : Exception(msg) { }
};

class OpenFailedException : public Exception
{
public:
  OpenFailedException() { }
  OpenFailedException(const std::string& msg) : Exception(msg) { }
};

class NotOpenException : public Exception
{
public:
  NotOpenException() { }
  NotOpenException(const std::string& msg) : Exception(msg) { }
};

class EOFException : public Exception
{
public:
  EOFException() { }
  EOFException(const std::string& msg) : Exception(msg) { }
};

class ReadFailedException : public Exception
{
public:
  ReadFailedException() { }
  ReadFailedException(const std::string& msg) : Exception(msg) { }
};

class WriteFailedException : public Exception
{
public:
  WriteFailedException() { }
  WriteFailedException(const std::string& msg) : Exception(msg) { }
};

class UnsupportedFormatException : public Exception
{
public:
  UnsupportedFormatException() { }
  UnsupportedFormatException(const std::string& msg) : Exception(msg) { }
};


// Monitor exceptions

class UnrecognizedSourceException : public Exception
{
public:
  UnrecognizedSourceException() { }
  UnrecognizedSourceException(const std::string& msg) : Exception(msg) { }
};

class MonitorFailedException : public Exception
{
public:
  MonitorFailedException() { }
  MonitorFailedException(const std::string& msg) : Exception(msg) { }
};


// Pipeline Dispatcher Exceptions

class UnregisteredFormatException : public Exception
{
public:
  UnregisteredFormatException() { }
  UnregisteredFormatException(const std::string& msg) : Exception(msg) { }
};

#endif // EXCEPTIONS_H
