#ifndef FILESOURCE_H
#define FILESOURCE_H

#include <string>
#include "Source.h"

class FileSource : public Source
{
public:
  enum
  {
    NONE   = 0x00,
    BINARY = 0x01,
    CREAT  = 0x02,
    EXCL   = 0x04,
    TRUNC  = 0x08
  };
protected:
  std::string filename_;

  int fd_;

  // Allows specification of custom block size, defaults to RawFormat::FILE_BUFFER
  unsigned int blockSize_;

#ifdef WIN32
  Handle win32event_;
#endif

protected:
  void openFile(Mode mode, const std::string& filename, unsigned int filemode);

public:
  // IN, OUT, and INOUT constructor
  FileSource(Mode mode, const std::string& filename);

  // OUT and INOUT constructor with create or append option
  FileSource(Mode mode, const std::string& filename, unsigned int filemode);

  // IN, OUT, and INOUT constructor
  FileSource(Mode mode, unsigned int blockSize, const std::string& filename);

  // OUT and INOUT constructor with create or append option
  FileSource(Mode mode, unsigned int blockSize, const std::string& filename, unsigned int filemode);

  ~FileSource();

  unsigned int getBlockSize() const { return blockSize_; }

  std::string getFilename() const { return filename_; }

  virtual void close();

  virtual Format* readRaw();

  virtual void writeRaw(const Format* fmt);
};

#endif // FILESOURCE_H
