#ifndef FORMAT_H
#define FORMAT_H

#include <stdlib.h>
#include "FormatVisitor.h"

class Creator;
class Source;

class Format
{
private:
  // Format type relative to a single source - zero is reserved for raw types
  unsigned int type_;

  Creator* creator_;

protected:
  // Source which generated the data
  Source* source_;

protected:
  // Regular constructor
  Format(unsigned int type, Creator* creator) : type_(type), creator_(creator), source_(NULL) { }

  // Constructor for copying
  Format(const Format& copy, Creator* creator) : type_(copy.type_), creator_(creator), source_(copy.source_) { }

public:
  virtual ~Format() { }

  unsigned int getType() const { return type_; }

  Creator* getCreator() const { return creator_; }

  Source* getSource() const { return source_; }

  // Initially source was to be private and only set by creators, but there are too many creators (source, converter, classifier)
  void setSource(Source* src) { source_ = src; }

  // Over-ride this in every derived format and change the return typename to match the derived typename
  // Creator is the object that is coloning the format
  virtual Format* clone(Creator* creator) const = 0;

  // Accept function for visitor
  virtual void accept(FormatVisitor& visitor) { visitor.visit(this); }

  // Accept function for visitor; use const when performing operations wihtin read only mutex lock
  virtual void accept(ConstFormatVisitor& visitor) const { visitor.visit(this); }
};

#endif // FORMAT_H
