#ifndef FORMATGROUP_H
#define FORMATGROUP_H

#include <vector>
#include "Format.h"
#include "PoolAllocator.h"

class FormatGroup : public Format
{
protected:
  typedef std::vector<Format*> Group;

  Group group_;

public:
  FormatGroup(Creator* creator) : Format(1, creator) { }

  FormatGroup(const FormatGroup& copy, Creator* creator);

  FormatGroup* clone(Creator* creator) const { return new FormatGroup(*this, creator); }

  ~FormatGroup()
  {
    for(Group::iterator iter = group_.begin(); iter != group_.end(); ++iter)
    {
      delete *iter;
    }
  }

  POOL_DECLARE(FormatGroup)

  size_t getNumFormats() const { return group_.size(); }

  void addFormat(Format* fmt) { group_.push_back(fmt); }

  void removeFormat(size_t pos);

  Format* getFormat(size_t pos) const { return group_[pos]; }
};

/*
#include <vector>
#include <deque>
#include <list>
#include "Format.h"
#include "PoolAllocator.h"

typedef std::vector<Format*> FormatGroupVectorContainer;
typedef std::deque<Format*> FormatGroupDequeContainer;
typedef std::list<Format*> FormatGroupListContainer;

template <typename Container = FormatGroupVectorContainer>
class FormatGroup : public Format
{
protected:
  typedef Container Group;

  Group group_;

public:
  FormatGroup(Creator* creator) : Format(1, creator) { }

  FormatGroup(const FormatGroup& copy, Creator* creator);

  FormatGroup* clone(Creator* creator) const { return new FormatGroup(*this, creator); }

  POOL_DECLARE(FormatGroup)

  size_t getNumFormats() const { return group_.size(); }

  void addFormat(Format* fmt) { group_.push_back(fmt); }

  void removeFormat(size_t pos);

  Format* getFormat(size_t pos) const;
};

template <typename Container>
POOL_DEFINE(FormatGroup<Container>, 1024)

// Type definitions for the FormatGroup with its three implicitly supported sequence containers
typedef FormatGroup<> FormatGroupVector;
typedef FormatGroup<FormatGroupDequeContainer> FormatGroupDeque;
typedef FormatGroup<FormatGroupListContainer> FormatGroupList;
*/

#endif // FORMATGROUP_H
