#ifndef FORMATVISITOR_H
#define FORMATVISITOR_H

class Format;

// Visitor for read/write locks
class FormatVisitor
{
public:
  virtual ~FormatVisitor() { }

  virtual void visit(Format*) = 0;
};

// Visitor for read-only locks
class ConstFormatVisitor
{
public:
  virtual ~ConstFormatVisitor() { }

  virtual void visit(const Format*) = 0;
};

// Decorator to adapt an existing function to the FormatVisitor
template <typename Func>
class FormatVisitorAdapter : public FormatVisitor
{
protected:
  Func func;
public:
  FormatVisitorAdapter(Func f) : func(f) { }

  virtual void visit(Format* format) { func(format); }
};

// Decorator to adapt an existing function to the FormatVisitor
template <typename Func>
class ConstFormatVisitorAdapter : public ConstFormatVisitor
{
protected:
  Func func;
public:
  ConstFormatVisitorAdapter(Func f) : func(f) { }

  virtual void visit(const Format* format) { func(format); }
};

#endif // FORMATVISITOR_H
