#ifndef FUNCTORTERMINATOR_H
#define FUNCTORTERMINATOR_H

#include <algorithm>
#include <vector>
#include "Format.h"
#include "PipelineFunctions.h"
#include "Pipeline.h"
#include "Source.h"

struct FunctorTerminatorFunction
{
public:
  virtual ~FunctorTerminatorFunction() { }

  virtual void operator()(Format* format) = 0;
};

// Pipeline terminator responsible for executing a functor with
// the data from the  pipeline.  Provides a general method for
// importing data to other components of an application.
class FunctorTerminator : public Terminator
{
protected:
  typedef std::vector< FunctorTerminatorFunction* > FunctionList;

  FunctionList functions_;

public:
  // Constructor
  FunctorTerminator(bool is_serial) : Terminator(is_serial) { }

  void addFunction(FunctorTerminatorFunction* function) { functions_.push_back(function); }

  void removeFunction(FunctorTerminatorFunction* function)
  {
    FunctionList::iterator iter = std::find(functions_.begin(), functions_.end(), function);

    if(iter != functions_.end())
    {
      functions_.erase(iter);
    }
  }

  FunctorTerminatorFunction* getFunction(size_t pos) const { return functions_[pos]; }

  void clear() { functions_.clear(); }

  virtual bool operator()(Format* format)
  {
    for(FunctionList::iterator iter = functions_.begin(); iter != functions_.end(); ++iter)
    {
      FunctorTerminatorFunction* f = *iter;
      (*f)(format);
    }

    return true;
  }
};

#endif // FUNCTORTERMINATOR_H

