#ifndef LRUCACHE_H
#define LRUCACHE_H

#include <deque>
#include "Cache.h"

class Format;

// Items placed in cache are copies
// May not be safe to modify values in cache
class LRUCache : public Cache
{
private:
  typedef std::deque<Format*> FormatCache;

  FormatCache cache_;

protected:
  /// Get an item from the cache
  virtual Format* get(size_t pos);

  /// Resize the cache if it exceeds the max size
  virtual void resize(size_t maxsize);

public:
  // Constructor
  LRUCache(size_t maxsize) : Cache(maxsize) { }

  /// Retrieve size of the cache
  virtual size_t getSize() const;

  /// Add a message to the cache
  virtual void addData(const Format* fmt);

  /// Visit items in the cache
  virtual void visit(FormatVisitor& visitor);

  /// Visit items in the cache
  virtual void visit(ConstFormatVisitor& visitor) const;
};

#endif // LRUCACHE_H
