#ifndef LRUMULTICACHE_H
#define LRUMULTICACHE_H

#include <algorithm>
#include <map>
#include <deque>
#include "Cache.h"

class Format;

// Items placed in cache are copies
// May not be safe to modify values in cache
template <typename ID, typename GetID>
class LRUMultiCache : public Cache
{
private:
  typedef std::deque<Format*> FormatCache;

  // Dictionary of format caches keyed by an ID of type T
  typedef std::map<ID, FormatCache> MultiCache;

  // The group of caches keyed by an ID unique to a format object
  MultiCache cache_;

  // The ID of the cache to be processed for the query functions: getSize, getData, visit
  ID queryCache_;

protected:
  /// Get an item from the cache
  virtual Format* get(size_t pos);

  /// Resize the cache if it exceeds the max size
  virtual void resize(size_t maxsize);

public:
  /// Constructor
  LRUMultiCache(size_t maxsize) : Cache(maxsize)
  {
    // Fill the queryCache_ with zeros
    char* ptr = static_cast<char*>(&queryCache_);
    std::fill(ptr, ptr + sizeof(queryCache_), 0);
  }

  /// Set the id of the cache to be processed by the query functions: getSize, getData, visit
  void setQueryCache(ID id);

  /// Retrieve size of the cache associated with the specified ID
  virtual size_t getSize() const;

  /// Add a message to the cache
  virtual void addData(const Format* fmt);

  /// Visit items in the cache; for read/write locking
  virtual void visit(FormatVisitor& visitor);

  /// Visit items in the cache; for read only locking
  virtual void visit(ConstFormatVisitor& visitor) const;
};

#endif // LRUMULTICACHE_H
