#include <tbb/parallel_for.h>
#include <tbb/partitioner.h>
#include "PipelineDispatcher.h"

class PipelineGroupForOp
{
protected:
  size_t limit_;
  size_t max_num_live_pipelines_;
public:
  PipelineGroupForOp(size_t limit, size_t max_num_live_pipelines) : limit_(limit), max_num_live_pipelines_(max_num_live_pipelines) { }

  void operator()(PipelineGroup::range_type& r) const
  {
    for(PipelineGroup::iterator p = r.begin(); p != r.end(); ++p)
    {
      if(p->second->getDataSize() > 0)
      {
        p->second->run(limit_, max_num_live_pipelines_);
      }
    }
  }
};

class PipelinesForOp
{
protected:
  Pipelines* pipelines_;
  size_t limit_;
  size_t max_num_live_pipelines_;
public:
  PipelinesForOp(Pipelines* pipelines, size_t limit, size_t max_num_live_pipelines) : pipelines_(pipelines), limit_(limit), max_num_live_pipelines_(max_num_live_pipelines) { }

  void operator()(const tbb::blocked_range<size_t>& r) const
  {
    for(size_t i = r.begin(); i != r.end(); ++i)
    {
      if((*pipelines_)[i]->getDataSize() > 0)
      {
        (*pipelines_)[i]->run(limit_, max_num_live_pipelines_);
      }
    }
  }
};

class PipelinesGroupForOp
{
protected:
  size_t limit_;
  size_t max_num_live_pipelines_;
public:
  PipelinesGroupForOp(size_t limit, size_t max_num_live_pipelines) : limit_(limit), max_num_live_pipelines_(max_num_live_pipelines) { }

  void operator()(PipelinesGroup::range_type& r) const
  {
    for(PipelinesGroup::iterator p = r.begin(); p != r.end(); ++p)
    {
      tbb::parallel_for(tbb::blocked_range<size_t>(0, p->second.size()), PipelinesForOp(&p->second, limit_, max_num_live_pipelines_));
    }
  }
};

void PipelineGroupDelegate::run(size_t limit, size_t max_num_live_pipelines)
{
  PipelineGroup::range_type r = group_->range(max_num_live_pipelines);
  tbb::parallel_for(r, PipelineGroupForOp(limit, max_num_live_pipelines));
}

void PipelinesDelegate::run(size_t limit, size_t max_num_live_pipelines)
{
  tbb::parallel_for(tbb::blocked_range<size_t>(0, pipelines_->size()), PipelinesForOp(pipelines_, limit, max_num_live_pipelines));
}

void PipelinesGroupDelegate::run(size_t limit, size_t max_num_live_pipelines)
{
  PipelinesGroup::range_type r = group_->range(max_num_live_pipelines);
  tbb::parallel_for(r, PipelinesGroupForOp(limit, max_num_live_pipelines));
}


void PipelineDispatcher::PipelineDispatcherForOp::operator()(PipelineDispatcher::Delegates::range_type& r) const
{
  for(Delegates::iterator d = r.begin(); d != r.end(); ++d)
  {
    d->second->run(32, 8);
  }
}

// One dispatch method - just constantly execute all - executes a lot of unnecessary code
void PipelineDispatcher::run()
{
  running_ = 1;
  while(running_)
  {
    Delegates::range_type r = delegates_.range(4);
    tbb::parallel_for(r, PipelineDispatcherForOp());
  }
}
