#ifndef RAWFORMAT_H
#define RAWFORMAT_H

#include <vector>
#include "PoolAllocator.h"
#include "Format.h"

class RawFormat : public Format
{
public:
  enum BufferSize
  {
    FILE_BUFFER = 4096,
    ETHERNET_BUFFER = 1500,
    DATAGRAM_BUFFER = 65535
  };

public:
  typedef std::vector<char> Buffer;

protected:
  Buffer buffer_;

  size_t dataSize_;

public:
  // Constructor - format type is 0 for raw types
  RawFormat(size_t size, Creator* creator) : Format(0, creator), buffer_(size), dataSize_(0) { }

  // Copy Constructor
  RawFormat(const RawFormat& copy, Creator* creator) : Format(copy, creator), buffer_(copy.buffer_.begin(), copy.buffer_.end()), dataSize_(copy.dataSize_) { }

  RawFormat* clone(Creator* creator) const { return new RawFormat(*this, creator); }

  POOL_DECLARE(RawFormat)

  char* getBuffer() { return &buffer_[0]; }

  const char* getBuffer() const { return &buffer_[0]; }

  size_t getBufferSize() const { return buffer_.size(); }

  void setBufferSize(size_t size) { buffer_.resize(size); }

  size_t getDataSize() const { return dataSize_; }

  void setDataSize(size_t size) { dataSize_ = size; }
};

#endif // RAWFORMAT_H
