#ifndef SOURCE_H
#define SOURCE_H

#include <tbb/concurrent_queue.h>
#include "Creator.h"

class Cache;
class Format;
class Classifier;

class Source : public Creator
{
public:
  enum Mode
  {
    IN    = 0,
    OUT   = 1,
    INOUT = 2
  };

  enum State
  {
    INACTIVE = 0,
    ACTIVE   = 1,
    DONE     = 2,
    ERROR    = 3
  };

#ifdef WIN32
//typedef HANDLE Handle;
#else
#define INVALID_HANDLE -1
typedef int Handle;
#endif

protected:
  typedef tbb::concurrent_queue< Format* > WriteQueue;

protected:
  Mode mode_;

  State state_;

  Handle readHandle_;

  Handle writeHandle_;

  Classifier* classifier_;

  Cache* readCache_;

  Cache* writeCache_;

  WriteQueue writeQueue_;

public:
  Source(Mode mode) : mode_(mode), state_(INACTIVE), readHandle_(INVALID_HANDLE), writeHandle_(INVALID_HANDLE), classifier_(NULL), readCache_(NULL), writeCache_(NULL) { }

  virtual ~Source() { }

  Mode getMode() const { return mode_; }

  State getState() const { return state_; }

  Handle getReadHandle() const;

  Handle getWriteHandle() const;

  int getWriteQueueSize() const { return writeQueue_.size(); }

  void setClassifier(Classifier* classifier) { classifier_ = classifier; }

  Classifier* getClassifier() const { return classifier_; }

  void setReadCache(Cache* cache);

  Cache* getReadCache() const;

  void setWriteCache(Cache* cache);

  Cache* getWriteCache() const;

  virtual void close() = 0;

  virtual Format* readRaw() = 0;

  virtual void writeRaw(const Format* fmt) = 0;

  virtual Format* read();

  virtual void write(const Format* fmt);

  virtual void writeQueued();

  virtual void enqueueForWrite(Format* fmt);
};

#endif
