#include <algorithm>
#include <vector>
#include "Exceptions.h"
#include "RawFormat.h"
#include "FormatGroup.h"
#include "TextLineFormat.h"
#include "TextLineClassifier.h"

Format* TextLineClassifier::toRaw(const Format* fmt)
{
  std::vector<std::string> lines;
  size_t size = 0;

  const TextLineFormat* line;
  const FormatGroup* group;

  // Process a group of lines or a single line
  if((line = dynamic_cast< const TextLineFormat* >(fmt)) != NULL)
  {
    const std::string& s = line->getLine();
    size = s.length();
    lines.push_back(s);
  }
  else if((group = dynamic_cast< const FormatGroup* >(fmt)) != NULL)
  {
    for(size_t i = 0; i < group->getNumFormats(); i++)
    {
      line = dynamic_cast< const TextLineFormat* >(group->getFormat(i));
      if(line == NULL)
      {
        throw UnsupportedFormatException("Invalid format specified for TextLineClassifier; TextLineClassifier only accepts TextLineFormat, or a FormatGroup containing TextLineFormat, for conversion to RawFormat");
      }

      const std::string& s = line->getLine();
      size += s.length();
      lines.push_back(s);
    }
  }
  else
  {
    throw UnsupportedFormatException("Invalid format specified for TextLineClassifier; TextLineClassifier only accepts TextLineFormat, or a FormatGroup containing TextLineFormat, for conversion to RawFormat");
  }

  RawFormat* rawfmt = new RawFormat(size, this);

  for(size_t i = 0; i < lines.size(); i++)
  {
    const std::string& s = lines[i];
    std::copy(s.begin(), s.end(), rawfmt->getBuffer());
  }

  rawfmt->setDataSize(size);

  return rawfmt;
}

// Produce a group of lines
Format* TextLineClassifier::fromRaw(const Format* fmt)
{
  const RawFormat* rawfmt = dynamic_cast< const RawFormat* >(fmt);
  if(rawfmt == NULL)
  {
    throw UnsupportedFormatException("Invalid format specified for TextLineClassifier; TextLineClassifier only accepts RawFormat for conversion to a FromatGroup containing TextLineFormat");
  }

  FormatGroup* group = NULL;

  const char* data = rawfmt->getBuffer();

  for(size_t i = 0; i < rawfmt->getDataSize(); i++)
  {
    buffer_.push_back(data[i]);
    if(data[i] == '\n')
    {
      if(group == NULL)
      {
        group = new FormatGroup(this);
      }

      group->addFormat(new TextLineFormat(buffer_, this));

      buffer_.clear();
    }
  }

  return group;
}
