#include <algorithm>
#include "Exceptions.h"
#include "RawFormat.h"
#include "TextLinesFormat.h"
#include "TextLinesClassifier.h"

Format* TextLinesClassifier::toRaw(const Format* fmt)
{
  const TextLinesFormat* lines = dynamic_cast< const TextLinesFormat* >(fmt);
  if(lines == NULL)
  {
    throw UnsupportedFormatException("Invalid format specified for TextLinesClassifier; TextLinesClassifier only accepts TextLinesFormat for conversion to RawFormat");
  }

  RawFormat* rawfmt = new RawFormat(lines->getSize(), this);

  for(size_t i = 0; i < lines->getNumLines(); i++)
  {
    const std::string& s = lines->getLine(i);
    std::copy(s.begin(), s.end(), rawfmt->getBuffer());
  }

  rawfmt->setDataSize(lines->getSize());

  return rawfmt;
}

Format* TextLinesClassifier::fromRaw(const Format* fmt)
{
  const RawFormat* rawfmt = dynamic_cast< const RawFormat* >(fmt);
  if(rawfmt == NULL)
  {
    throw UnsupportedFormatException("Invalid format specified for TextLinesClassifier; TextLinesClassifier only accepts RawFormat for conversion to TextLinesFormat");
  }

  TextLinesFormat* lines = NULL;

  const char* data = rawfmt->getBuffer();

  for(size_t i = 0; i < rawfmt->getDataSize(); i++)
  {
    buffer_.push_back(data[i]);
    if(data[i] == '\n')
    {
      if(lines == NULL)
      {
        lines = new TextLinesFormat(this);
      }

      lines->addLine(buffer_);

      buffer_.clear();
    }
  }

  return lines;
}
