#ifndef TEXTLINESFORMAT_H
#define TEXTLINESFORMAT_H

#include <algorithm>
#include <deque>
#include <string>
#include "PoolAllocator.h"
#include "Format.h"

class TextLinesFormat : public Format
{
protected:
  typedef std::deque<std::string> Lines;

  Lines lines_;

  size_t bytes_;

public:
  TextLinesFormat(Creator* creator) : Format(0, creator), bytes_(0) { }

  TextLinesFormat(const TextLinesFormat& copy, Creator* creator) : Format(copy, creator), lines_(copy.lines_.begin(), copy.lines_.end()), bytes_(bytes_) { }

  TextLinesFormat* clone(Creator* creator) const { return new TextLinesFormat(*this, creator); }

  POOL_DECLARE(TextLinesFormat)

  size_t getNumLines() const { return lines_.size(); }

  size_t getSize() const { return bytes_; }

  std::string getLine(size_t pos) const { return lines_[pos]; }

  void addLine(const std::string& line)
  {
    lines_.push_back(line);
    bytes_ += line.length();
  }

  void removeLine(size_t pos)
  {
    Lines::iterator iter = lines_.begin();
    std::advance(iter, pos);
    bytes_ -= (*iter).length();
    lines_.erase(iter);
  }
};

#endif // TEXTLINESFORMAT_H
