#ifndef UNICASTSOURCE_H
#define UNICASTSOURCE_H

#include <string>
#include "sockdefs.h"
#include "Source.h"

class IPEndPoint
{
protected:
  std::string addr_;

  unsigned short port_;

public:
  IPEndPoint() : port_(0) { }

  IPEndPoint(const std::string& addr, unsigned short port) : addr_(addr), port_(port) { }

  IPEndPoint(const IPEndPoint& endpoint) : addr_(endpoint.addr_), port_(endpoint.port_) { }

  std::string getAddress() const { return addr_; }

  void setAddress(const std::string& addr) { addr_ = addr; }

  unsigned short getPort() const { return port_; }

  void setPort(unsigned short port) { port_ = port; }
};

class UnicastSource : public Source
{
protected:
  std::string addr_;

  unsigned short port_;

  SOCKET sockfd_;

  SOCKADDR_IN destination_;

  bool attached_;

protected:
  void create();

  void bind(const IPEndPoint& ie);

public:
  // OUT Constructor
  UnicastSource(Mode mode);

  // OUT Constructor
  UnicastSource(Mode mode, const std::string& addr, unsigned short port);

  // IN or INOUT Constructor
  UnicastSource(Mode mode, unsigned short port);

  // IN or INOUT Constructor
  UnicastSource(Mode mode, const IPEndPoint& endpoint);

  // INOUT Constructor
  UnicastSource(Mode mode, unsigned short localport, const std::string& addr, unsigned short port);

  // INOUT Constructor
  UnicastSource(Mode mode, IPEndPoint, const std::string& addr, unsigned short port);

  ~UnicastSource();

  unsigned short getPort() const { return port_; }

  std::string getAddress() const { return addr_; }

  virtual void close();

  virtual Format* readRaw();

  virtual void writeRaw(const Format* fmt);
};

#endif // UNICASTSOURCE_H
