/********************************************************************************
*                     S o c k e t  C o m p a t i b i l i t y                    *
*********************************************************************************
* Copyright (C) 1999,2000,2001,2008 by Dustin Graves.   All Rights Reserved.    *
********************************************************************************/

#ifndef _SOCKDEFS_H_
#define _SOCKDEFS_H_

#ifndef WIN32
#include <errno.h>
#include <unistd.h>
#include <memory.h>
#include <string.h>
#include <sys/select.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <netinet/in.h>
#include <sys/ioctl.h>
#include <netdb.h>

#ifdef Solaris
#include <sys/filio.h>
#include <sys/sockio.h>
#endif


typedef int                 SOCKET;
typedef struct sockaddr     SOCKADDR;
typedef struct sockaddr_in  SOCKADDR_IN;
typedef char                TTL_T;
typedef unsigned long       u_long;

#define INVALID_SOCKET             (-1)
#define SOCKET_ERROR               (-1)

#define PLATERR(x) x
inline int closesocket(SOCKET sock) { return close(sock); }
inline int ioctlsocket(SOCKET sock, long cmd, u_long* argp) { return ioctl(sock, cmd, argp); }
inline void setSockErr(int32_t err) {errno=err;}
inline int32_t getSockErr()       {return errno;}

inline char* getSockErrString(char* buffer, size_t len)
{
  memset(buffer, 0, len);
  strncpy(buffer, strerror(errno), len - 1);
  return buffer;
}

inline bool initializeSocket()    {return true;}
inline bool finalizeSocket()      {return true;}

#else

#ifndef WIN32_LEAN_AND_MEAN
#define WIN32_LEAN_AND_MEAN
#endif

#include <windows.h>
#include <winsock2.h>
#include <ws2tcpip.h>

typedef int socklen_t;
typedef unsigned long in_addr_t;
typedef int TTL_T;

#define PLATERR(x) (WSA##x)              //Put our error value into proper WIN32 format
#define getSockErr  WSAGetLastError
#define setSockErr  WSASetLastError

inline char* getSockErrString(char* buffer, size_t len)
{
  memset(buffer, 0, len);
  FormatMessage(
      FORMAT_MESSAGE_FROM_SYSTEM|FORMAT_MESSAGE_IGNORE_INSERTS,
      NULL,
      WSAGetLastError(),
      MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
      (LPTSTR) buffer,
      len - 1,
      NULL
  );
  return buffer;
}

inline bool finalizeSocket()
{
  return (WSACleanup() == SOCKET_ERROR) ? false : true;
}

inline bool initializeSocket ()
{
  WSADATA stWSAData;
  return (WSAStartup(0x0202, &stWSAData) !=0 ) ? false : true;
}

#endif //WIN32

#ifndef SHUT_RD
#define SHUT_RD 0
#endif

#ifndef SHUT_WR
#define SHUT_WR 1
#endif

#ifndef SHUT_RDWR
#define SHUT_RDWR 2
#endif

#ifndef INADDR_NONE
#define INADDR_NONE 0xFFFFFFFF
#endif

#ifndef INET_ADDRSTRLEN
#define INET_ADDRSTRLEN 16
#endif

#define MAXSOCKADDR 128  //Max sockaddr structure size
#define EPHEMERAL_PORT 0
#define IN4ADDR_WILDCARD htonl(INADDR_ANY)

//WIN32 is missing an inet_aton function
#ifdef WIN32
inline int inet_aton(const char* strptr,struct in_addr* addrptr)
{
  uint32_t addr=inet_addr(strptr);
  if(addr!=INADDR_NONE)
  {
    addrptr->s_addr=addr;
    return 1;
  }
  return 0;
}
#endif

#endif
