#ifdef WIN32
#include <windows.h>
#define usleep(x) Sleep(x/1000)
#define TRETURN DWORD WINAPI
#else
#include <pthread.h>
#include <unistd.h>
#define TRETURN void*
#endif

#include <iostream>
#include <tbb/tick_count.h>
#include <tbb/task_scheduler_init.h>
#include "Exceptions.h"
#include "ForwardTerminator.h"
#include "PipelineDispatcher.h"
#include "Pipeline.h"
#include "Monitor.h"
#include "FileSource.h"

using namespace std;

TRETURN run_monitor(void* monitor)
{
  tbb::task_scheduler_init tsi;
  static_cast<Monitor*>(monitor)->run();
  return NULL;
}

TRETURN run_dispatcher(void* dispatcher)
{
  tbb::task_scheduler_init tsi;
  static_cast<PipelineDispatcher*>(dispatcher)->run();
  return NULL;
}

int main()
{
  tbb::task_scheduler_init tsi;

  tbb::tick_count start = tbb::tick_count::now();

  try
  {
    // Create input and output sources
    FileSource in(FileSource::IN, "input.txt");
    FileSource out(FileSource::OUT, "output.txt", FileSource::CREAT|FileSource::TRUNC);

    // Add the pipeline delegate for the source to the diaptcher
    ForwardTerminator term(true);
    Pipeline pipeline;
    PipelineDelegate delegate(&pipeline);

    PipelineDispatcher dispatch;
    dispatch.addSourceDelegate(&in, &delegate);

    // Add the source and its delegate to the monitor
    Monitor monitor;
    monitor.addSourceMonitor(&in, Monitor::READ, &delegate);
    monitor.addSourceMonitor(&out, Monitor::WRITE, NULL);

    // Setup the pipeline with a terminator to forward to the output source
    term.addSource(&out);
    pipeline.setTerminator(&term);

    // Start monitor and dispatcher threads
#ifdef WIN32
    HANDLE t1, t2;
    t1 = CreateThread(NULL, 0, run_monitor, (void*)&monitor, 0, NULL);
    t2 = CreateThread(NULL, 0, run_dispatcher, (void*)&dispatch, 0, NULL);
#else
    pthread_t t1, t2;
    pthread_create(&t1, NULL, run_monitor, (void*)&monitor);
    pthread_create(&t2, NULL, run_dispatcher, (void*)&dispatch);
#endif

    // Wait for eof
    while(monitor.getNumMonitoredSources() > 1)
    {
      ;
    }

    // Wait for writer
    while(out.getWriteQueueSize() > 0)
    {
      ;
    }

    monitor.stop();
    dispatch.stop();

#ifdef WIN32
    WaitForSingleObject(t1, INFINITE);
    WaitForSingleObject(t2, INFINITE);
#else
    pthread_join(t1, NULL);
    pthread_join(t2, NULL);
#endif
  }
  catch(Exception e)
  {
    std::cout << e.getMessage() << std::endl;
  }

  std::cout << "File copied in " << (tbb::tick_count::now() - start).seconds() << " seconds" << std::endl;

  return 0;
}
